/**
 *	Header file for Tarada Library
 *	Copyright (c) 2010 Tarada
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as published
 *	by the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef taradalib_h
#define taradalib_h

class TaradaLib
{
 private:

  // Internal Variables
  bool debug;
  // End Internal Variables

  /* Init */ void Init();
  /**
   * Description: Performs init operations common amongst all constructors
   */

  /* Tarada Library Error Reporter */ void Error(int, std::string, std::string, std::string, int);
  /**
   * Description: Reports Tarada Library errors
   * Arguments: Integer - Seriousness (from 1-9), String - Error message (appears onscreen and in logs), String - Source file name (appears onscreen and in logs) String - Calling function name (appears onscreen and in logs), Integer - Line number (appears on screen and in logs)
   */

 public:
  
  /* Tarada Library Constructor */ TaradaLib();
  /**
   * Description: Creates an instance of the Tarada Library
   */

  /* Set Debug */ void setDebug(bool);
  /**
   * Description: Sets internal debug variable, turning on or off debug mode
   * Arguments: Boolean - Value to set debug variable to
   */

  /* Compare Strings */ bool comparestrings(std::string, std::string);
  /**
   * Return Type: Boolean
   * Description: Checks each character of a string to see if it matches
   * Arguments: String - First string to compare; String - Second string to compare
   * Return Value: True - If the argument strings are identical; False - If the argument strings are not identical
   */

  /* Palindrome Checker */ bool palindromecheck(std::string); 
  /**
   * Return Type: Boolean
   * Description: Reverses a string and checks to see if the original and the reversed string match
   * Arguments: String - String to check
   * Return Value: True - If the original string and the reversed strings match; False - If the original string and the reversed string do not match
   */

  /* Factorial Calculator */ int factorial(int);
  /**
   * Return Type: Integer
   * Description: Computes the factorial of a number
   * Arguments: Integer - Integer to compute the factorial of
   * Return Value: Factorial of the argument
   */
  
  /* Random Number Generator */ int randomnum(int); 
  /**
   * Return Type: Integer
   * Description: Generates a random number
   * Arguments: Integer - The highest random number allowed
   * Return Value: A random number between 0 and the first argument
   */
  
  /* Random Number Generator */ int randomnum(int, int);
  /**
   * Return Type: Integer
   * Description: Generates a random number
   * Arguments: Integer - The highest random number allowed; Integer - The lowest random number allowed
   * Return Value: A random number berween the second argument and the first argument
   */

  /* Prime Number Checker */ bool primenumcheck(int);
  /**
   * Return Type: Boolean
   * Description: Checks if a number is prime or composite
   * Arguments: Integer - Integer to check
   * Return Value: True - If the integer is prime; False - If the integer is composite
   */

  /* Tarada Library Destructor */ ~TaradaLib();
  /**
   * Description: Destroys an instance of the Tarada Library
   */
};

/**
 * Usage:
 * #include <iostream>
 * #include <tarada/taradalib.h>
 * using namespace std;
 * int main()
 * {
 *   TaradaLib t; // Start Tarada Library
 *   char input;
 *
 *   cout <<"Do you want to turn on debug mode (y/n)? ";
 *   cin >>input;
 *   if (input='y' || input='Y')
 *     t.debug=true;
 *   else
 *     t.debug=false;
 *
 *   if ( t.palindromecheck("Madam, I'm Adam!") == true ) // Palindrome Checker Example
 *     cout <<"\"Madam, I'm Adam!\" is a palindrome!" <<endl;
 *   else
 *     cout <<"\"Madam, I'm Adam!\" is not a palindrome!" <<endl;
 *
 *
 *   cout <<"The factorial of 10 is " <<t.factorial(10) <<"!" <<endl; // Factorial Calculator Example
 *
 *   cout <<"Random number between 0 and 10: " <<t.randomnum(10) <<"!" <<endl; // Random Number Generator Example
 *
 *   return 0;
 * }
 */

/**
 * Compilation:
 *  To use Tarada Library in your C++ program, add "#include <tarada/taradalib.h>" to your program and compile it with:
 *   g++ -c progname.cpp `taradalibflags --headers-only` -o progname.o
 *   g++ progname.cpp `taradalibflags` -o progname.bin
 */

#endif /* taradalib_h */
