/**
 *	C++ Source File for Tarada Library
 *      Copyright (c) 2010 Tarada
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as published
 *	by the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include <iostream>
#include <fstream>
#include <vector>

#ifdef HAVE_STRING_H
#include <string>
#endif

#ifdef HAVE_CCTYPE_H
#include <cctype>
#endif

#ifdef STDC_HEADERS
#include <ctime>
#include <cstdlib>
#endif

#include "taradalib.h"
using namespace std;

// Internal Functions
void TaradaLib::Init()
{
  // Variable Initializations
  debug=false;
  // End Variable Initializations

  srand(time(NULL)); // Seed Random Number Generator
  system("/bin/mkdir -p /tmp/taradalib/log /tmp/taradalib/tmpfiles");
}

void TaradaLib::Error(int seriousness, string message, string file, string callingfunction, int linenum)
{
  // Variable Declarations
  ofstream logFile;
  string errorseriousness;
  // End Variable Declarations

  // Variable Initializations
  if (seriousness<=3)
    errorseriousness="minor error";
  else if (seriousness>3 && seriousness<=5)
    errorseriousness="serious error";
  else if (seriousness>5 && seriousness<=9)
    errorseriousness="fatal error";
  else
    errorseriousness="undefined error";
  // End Variable Initializations

  // Open Files
  logFile.open("/var/log/taradalib/taradalib.log", ios::app);
  // End Open Files

  logFile <<"taradalib: " <<errorseriousness <<". Error level " <<seriousness <<endl
	  <<"File: " <<file <<endl
	  <<"Function: " <<callingfunction <<endl
	  <<"Line: " <<linenum <<endl <<"Error message: "
	  <<message <<endl;
  cerr <<"taradalib: " <<errorseriousness <<". Error level " <<seriousness <<endl
       <<"File: " <<file <<endl
  <<"Function: " <<callingfunction <<endl
       <<"Error message: " <<message <<endl;

  // Close Files
  logFile.close();
  // End Close Files

  if (seriousness>5 && seriousness<=9)
    exit(seriousness);
}
// End Internal Functions

// Tarada Library Functions
TaradaLib::TaradaLib() // Constructor
{
  Init();
}

void TaradaLib::setDebug(bool input)
{
  debug=input;
}

bool TaradaLib::comparestrings(string input, string input2)
{
  bool same;
  if (input.size()==input2.size())
    {
      for (int i=0; i<=input.size(); i++)
	{
	  if (input[i]!=input2[i])
	    {
	      same=false;
	      if (debug==true)
		cout <<"\"" <<input <<"\" and \"" <<input2 <<"\" are not the same." <<endl;
	      break;
	    }
	  else
	    same=true;
	}
    }
  else
    same=false;
  if (debug==true && same==true)
    cout <<"\"" <<input <<"\" and \"" <<input2 <<"\" are the same." <<endl;
  return same;
}

bool TaradaLib::palindromecheck(std::string argpalindrome)
{

  // Variable Declarations
  vector<char> reversepalindrome(argpalindrome.size());
  int strlength=argpalindrome.size();
  int temp=strlength;
  int temp2;
  vector<char> palindrome(strlength);
  for (int i=0, j=0; i<temp; i++) // Converts input into an vector of characters, leaving out non-alphanumeric charcters
  {
    if ((argpalindrome.at(i) >= '0' && argpalindrome.at(i) <= '9') || 
	(argpalindrome.at(i) >= 'A' && argpalindrome.at(i) <= 'Z') ||
	(argpalindrome.at(i) >= 'a' && argpalindrome.at(i) <= 'z'))
    {
      palindrome.at(j)=tolower(argpalindrome.at(i));
      if (debug==true)
	cout <<"palindrome.at(" <<j <<") set to " <<tolower(argpalindrome.at(i)) <<"." <<endl;
      j++;
    }
    else
      strlength--;
  }
  bool strcomparison;
  // End Variable Declarations
  temp=strlength-1;
  for (int i=0; i<strlength; i++) // Reverses vector of characters and stores it in another vector of characters
  {
    reversepalindrome.at(i)=palindrome.at(temp);
    if (debug==true)
      cout <<"reversepalindrome.at(" <<i <<") set to " <<palindrome.at(temp) <<"." <<endl;
    temp--;
  }
  
  for (int i=0; i<strlength; i++) // Checks if vector<char> palindrome and vector<char> reversepalindrome are equal and sets return variable accordingly
  {
    if (reversepalindrome.at(i)==palindrome.at(i))
      strcomparison=true;
    else
    {
      strcomparison=false;
      break;
    }
  }

  if (debug==true && strcomparison==true)
    cout <<"Returning true." <<endl;
  else if (debug==true && strcomparison==false)
    cout <<"Returing false." <<endl;
  return strcomparison;

}

int TaradaLib::factorial(int input)
{

  int output=1;
  while (input>0)
  {
    output*=input;
    input--;
  }

  if (debug==true)
    cout <<"Returning " <<output <<"." <<endl;
  return output;

}

int TaradaLib::randomnum(int input)
{
  return rand() % (input+1); // Returns random number between 0 and input
}

int TaradaLib::randomnum(int input, int floor)
{
  return (rand() % (input+1-floor)) + floor; // Returns random number between floor and input
}

bool TaradaLib::primenumcheck(int j)
{
  // Variable Declarations
  bool flag;
  // End Variable Declarations
  flag=false;
  for (int i = 2; i < j; i++) // Look for factors
    {
      if ( j%i==0 ) // Check if the number is divisible by a number
	{
	  flag=false; // Conclude that it is not a prime number
	  break; // End the loop
	}
      else
	flag=true; // No conclusion yet
    }
  return flag;
}

TaradaLib::~TaradaLib() // Destructor
{
  system("/bin/rm -rf /tmp/taradalib");
}
// End Tarada Library Functions
